#! /usr/bin/env node

const path = require('path')
const fs = require('fs')
const shell = require('shelljs')
const colors = require('colors')
const clone = require('git-clone')
const cwdPath = process.cwd(); //需要获得命令执行的位置
const help = require('./helper')
const package = require('../package.json')

// const templatePath = path.resolve(__dirname, '../template')//靠背路径
// const readline = require('readline')//按行写入

const program = require('../utils/commander')


/**
 *
 * @param {string} path
 * @returns {boolean} 路径是否存在
 */
function fsExistsSync(path) {
    try {
        fs.accessSync(path, fs.F_OK)
    } catch (e) {
        return false
    }
    return true
}


/*
 * @param {string} projectName 
 */
function installDependencies(projectName) {
    console.log('\ninstalling node_modules...');
    if (shell.exec(`cd ${projectName} && npm install --registry=https://registry.npm.taobao.org`).code) {
        console.log(colors.red('install dependencies failed!'))
    } else {
        console.log(
            colors.blue(
                [
                    `Success! Created ${projectName} at ${cwdPath}.`,
                    'Inside that directory, you can run several commands and more:',
                    '  * npm run build: dll your project.',
                    '  * npm run dev: Run dev.',
                    'We suggest that you begin by typing:',
                    `  cd ${projectName}`,
                    '  npm run dev',
                    'Happy hacking!'
                ].join('\n')
            )
        );
    }

}


/**
 * @param {array} argvs 
 */
async function newProject(...argvs) {
    let [projectName, templateName] = argvs
    const targetPath = `${cwdPath}/${projectName}`;

    templateName = templateName ? templateName : 'react' //默认下载react项目模板
    try {
    	console.log('clone start...')
        clone(
            `https://github.com/zhaozeq/init_${templateName}_project.git`,
            `${targetPath}`,
            null,
            () => {
                console.log(colors.blue(`${templateName} project has cloned!`))
                shell.rm('-rf', `${targetPath}/.git`)
                installDependencies(projectName)
            })
    } catch (error) {
        console.log(error);
    }
}

try {
    //配置命令 option方法返回构造函数this 因此可以连写
    program
        .option('-h, --help', help)
        .option('-v, --version', () => {
            console.log(package.version)
        })

    program
        .command('new <project>')
        .action(newProject)
    program.catch((...argvs) => {
        console.log(colors.red(`Unknown use command univew-cli ${argvs}`));
        console.log('Run ' + colors.blue('univew-cli --help') + ' to get the Commands that dva-native-cli supports');
    })
    /*前两个参数为node所在地址和正在执行文件的地址，就是uniview-cli.js*/
    program.parse(process.argv) //process.argv:命令行下输入的参数
} catch (error) {
    console.log(error)
}